/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 09:28:03.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.examwork.api.controller;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.*;
import cn.com.qmth.examcloud.core.examwork.service.NoticeService;
import cn.com.qmth.examcloud.core.examwork.service.bean.NoticeInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.NoticeInfoQuery;
import cn.com.qmth.examcloud.core.examwork.service.bean.UserNoticeInfo;
import cn.com.qmth.examcloud.core.examwork.service.bean.UserNoticeInfoQuery;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import com.mysql.cj.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 公告类
 */
@RestController
@Api(tags = "公告类")
@RequestMapping("${$rmp.ctr.examwork}/notice")
public class NoticeController extends ControllerSupport {
    @Autowired
    private NoticeService noticeService;

    @ApiOperation(value = "分页查询消息列表", notes = "带条件带分页")
    @GetMapping("getPagedNoticeList/{curPage}/{pageSize}")
    public PageInfo<NoticeDomain> getPagedNoticeList(@PathVariable Integer curPage,
                                                     @PathVariable Integer pageSize, NoticeDomainQuery query) {

        NoticeInfoQuery infoQuery= new NoticeInfoQuery();
        PageInfo<NoticeInfo> pagedNoticeInfo =  noticeService.getPagedNoticeList(curPage,pageSize,infoQuery);
        // TOOD
        return null;
    }

    @GetMapping("/getUserNoticeList")
    @ApiOperation(value = "获取用户公告列表")
    public List<UserNoticeDomain> getUserNoticeList(UserNoticeDomainQuery query) {
        List<UserNoticeDomain> resultList = new ArrayList<>();
        UserNoticeInfoQuery noticeQuery = getNoticeQuery(query);

        List<UserNoticeInfo> noticeInfoList = noticeService.getNoticeList(noticeQuery);

        if (null != noticeInfoList && !noticeInfoList.isEmpty()) {
            resultList = getNoticeDomainList(noticeInfoList);
        }
        return resultList;
    }

    @PostMapping("/updateNoticeReadStatus")
    @ApiOperation(value = "更新通知状态为已读")
    public void updateNoticeReadStatus(@ApiParam(value = "通知id,多个以逗号分隔") @RequestParam(required = true) String noticeId) {
        if (StringUtils.isNullOrEmpty(noticeId)) {
            throw new StatusException("100001", "通知id不允许为空");
        }
        User user = this.getAccessUser();
        noticeService.updateNoticeReadStatus(noticeId, user.getUserType(), user.getUserId());
    }

    private UserNoticeInfoQuery getNoticeQuery(UserNoticeDomainQuery query) {
        User user = this.getAccessUser();
        UserNoticeInfoQuery noticeQuery = new UserNoticeInfoQuery();
        noticeQuery.setHasRead(query.getHasRead());
        noticeQuery.setRootOrgId(user.getRootOrgId());
        noticeQuery.setUserId(user.getUserId());
        noticeQuery.setUserType(user.getUserType());
        return noticeQuery;
    }

    private List<UserNoticeDomain> getNoticeDomainList(List<UserNoticeInfo> noticeInfoList) {
        List<UserNoticeDomain> resultList = new ArrayList<>();
        for (UserNoticeInfo info : noticeInfoList) {
            UserNoticeDomain domain = new UserNoticeDomain();
            domain.setContent(info.getContent());
            domain.setId(info.getId());
            domain.setPublisher(info.getPublisher());
            domain.setPublishTime(info.getPublishTime());
            domain.setHasRead(info.getHasRead());
            domain.setTitle(info.getTitle());
            resultList.add(domain);
        }
        return resultList;
    }
}